@@ -1,38 +1,21 @@
 /*
- * JBoss, Home of Professional Open Source.
- * Copyright 2016, Red Hat, Inc., and individual contributors
- * as indicated by the @author tags. See the copyright.txt file in the
- * distribution for a full listing of individual contributors.
- *
- * This is free software; you can redistribute it and/or modify it
- * under the terms of the GNU Lesser General Public License as
- * published by the Free Software Foundation; either version 2.1 of
- * the License, or (at your option) any later version.
- *
- * This software is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this software; if not, write to the Free
- * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
- * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
+ * Copyright The WildFly Authors
+ * SPDX-License-Identifier: Apache-2.0
  */
 
 package org.jboss.as.clustering.infinispan.subsystem.remote;
 
 import java.util.ArrayList;
-import java.util.Collections;
 import java.util.EnumMap;
 import java.util.HashMap;
 import java.util.List;
 import java.util.Map;
-import java.util.NoSuchElementException;
 import java.util.Properties;
+import java.util.concurrent.TimeUnit;
 import java.util.function.Consumer;
 import java.util.function.Function;
 import java.util.function.Supplier;
+import java.util.stream.Collectors;
 
 import javax.management.MBeanServer;
 
@@ -43,16 +26,13 @@ import org.infinispan.client.hotrod.configuration.ConfigurationBuilder;
 import org.infinispan.client.hotrod.configuration.ConnectionPoolConfiguration;
 import org.infinispan.client.hotrod.configuration.ExecutorFactoryConfiguration;
 import org.infinispan.client.hotrod.configuration.SecurityConfiguration;
-import org.infinispan.client.hotrod.configuration.TransactionConfiguration;
 import org.infinispan.commons.marshall.Marshaller;
 import org.infinispan.commons.util.AggregatedClassLoader;
 import org.jboss.as.clustering.controller.CapabilityServiceNameProvider;
 import org.jboss.as.clustering.controller.CommonRequirement;
 import org.jboss.as.clustering.controller.CommonUnaryRequirement;
 import org.jboss.as.clustering.controller.ResourceServiceConfigurator;
-import org.jboss.as.clustering.dmr.ModelNodes;
-import org.jboss.as.clustering.infinispan.InfinispanLogger;
-import org.jboss.as.clustering.infinispan.MBeanServerProvider;
+import org.jboss.as.clustering.infinispan.jmx.MBeanServerProvider;
 import org.jboss.as.clustering.infinispan.subsystem.ThreadPoolResourceDefinition;
 import org.jboss.as.clustering.infinispan.subsystem.remote.RemoteCacheContainerResourceDefinition.Attribute;
 import org.jboss.as.controller.OperationContext;
@@ -64,16 +44,12 @@ import org.jboss.as.network.OutboundSocketBinding;
 import org.jboss.as.server.Services;
 import org.jboss.dmr.ModelNode;
 import org.jboss.dmr.Property;
-import org.jboss.marshalling.ModularClassResolver;
 import org.jboss.modules.Module;
 import org.jboss.modules.ModuleLoader;
 import org.jboss.msc.Service;
 import org.jboss.msc.service.ServiceBuilder;
 import org.jboss.msc.service.ServiceController;
 import org.jboss.msc.service.ServiceTarget;
-import org.wildfly.clustering.infinispan.marshalling.jboss.JBossMarshaller;
-import org.wildfly.clustering.infinispan.marshalling.protostream.ProtoStreamMarshaller;
-import org.wildfly.clustering.marshalling.protostream.ModuleClassLoaderMarshaller;
 import org.wildfly.clustering.service.CompositeDependency;
 import org.wildfly.clustering.service.Dependency;
 import org.wildfly.clustering.service.FunctionalService;
@@ -94,19 +70,18 @@ public class RemoteCacheContainerConfigurationServiceConfigurator extends Capabi
     private final Properties properties = new Properties();
     private final SupplierDependency<ConnectionPoolConfiguration> connectionPool;
     private final SupplierDependency<SecurityConfiguration> security;
-    private final SupplierDependency<TransactionConfiguration> transaction;
 
     private volatile SupplierDependency<MBeanServer> server;
     private volatile int connectionTimeout;
     private volatile String defaultRemoteCluster;
-    private volatile int keySizeEstimate;
     private volatile int maxRetries;
     private volatile String protocolVersion;
     private volatile int socketTimeout;
     private volatile boolean tcpNoDelay;
     private volatile boolean tcpKeepAlive;
-    private volatile int valueSizeEstimate;
     private volatile boolean statisticsEnabled;
+    private volatile long transactionTimeout;
+    private volatile HotRodMarshallerFactory marshallerFactory;
 
     RemoteCacheContainerConfigurationServiceConfigurator(PathAddress address) {
         super(RemoteCacheContainerResourceDefinition.Capability.CONFIGURATION, address);
@@ -115,21 +90,20 @@ public class RemoteCacheContainerConfigurationServiceConfigurator extends Capabi
         this.modules = new ServiceSupplierDependency<>(RemoteCacheContainerComponent.MODULES.getServiceName(address));
         this.connectionPool = new ServiceSupplierDependency<>(RemoteCacheContainerComponent.CONNECTION_POOL.getServiceName(address));
         this.security = new ServiceSupplierDependency<>(RemoteCacheContainerComponent.SECURITY.getServiceName(address));
-        this.transaction = new ServiceSupplierDependency<>(RemoteCacheContainerComponent.TRANSACTION.getServiceName(address));
     }
 
     @Override
     public ServiceConfigurator configure(OperationContext context, ModelNode model) throws OperationFailedException {
         this.connectionTimeout = Attribute.CONNECTION_TIMEOUT.resolveModelAttribute(context, model).asInt();
         this.defaultRemoteCluster = Attribute.DEFAULT_REMOTE_CLUSTER.resolveModelAttribute(context, model).asString();
-        this.keySizeEstimate = Attribute.KEY_SIZE_ESTIMATE.resolveModelAttribute(context, model).asInt();
         this.maxRetries = Attribute.MAX_RETRIES.resolveModelAttribute(context, model).asInt();
         this.protocolVersion = Attribute.PROTOCOL_VERSION.resolveModelAttribute(context, model).asString();
         this.socketTimeout = Attribute.SOCKET_TIMEOUT.resolveModelAttribute(context, model).asInt();
         this.tcpNoDelay = Attribute.TCP_NO_DELAY.resolveModelAttribute(context, model).asBoolean();
         this.tcpKeepAlive = Attribute.TCP_KEEP_ALIVE.resolveModelAttribute(context, model).asBoolean();
-        this.valueSizeEstimate = Attribute.VALUE_SIZE_ESTIMATE.resolveModelAttribute(context, model).asInt();
         this.statisticsEnabled = Attribute.STATISTICS_ENABLED.resolveModelAttribute(context, model).asBoolean();
+        this.transactionTimeout = Attribute.TRANSACTION_TIMEOUT.resolveModelAttribute(context, model).asLong();
+        this.marshallerFactory = HotRodMarshallerFactory.valueOf(Attribute.MARSHALLER.resolveModelAttribute(context, model).asString());
 
         this.clusters.clear();
 
@@ -148,7 +122,7 @@ public class RemoteCacheContainerConfigurationServiceConfigurator extends Capabi
         this.server = context.hasOptionalCapability(CommonRequirement.MBEAN_SERVER.getName(), null, null) ? new ServiceSupplierDependency<>(CommonRequirement.MBEAN_SERVER.getServiceName(context)) : null;
 
         this.properties.clear();
-        for (Property property : ModelNodes.optionalPropertyList(Attribute.PROPERTIES.resolveModelAttribute(context, model)).orElse(Collections.emptyList())) {
+        for (Property property : Attribute.PROPERTIES.resolveModelAttribute(context, model).asPropertyListOrEmpty()) {
             this.properties.setProperty(property.getName(), property.getValue().asString());
         }
 
@@ -158,7 +132,7 @@ public class RemoteCacheContainerConfigurationServiceConfigurator extends Capabi
     @Override
     public ServiceBuilder<?> build(ServiceTarget target) {
         ServiceBuilder<?> builder = target.addService(this.getServiceName());
-        Consumer<Configuration> configuration = new CompositeDependency(this.loader, this.modules, this.connectionPool, this.security, this.transaction, this.server).register(builder).provides(this.getServiceName());
+        Consumer<Configuration> configuration = new CompositeDependency(this.loader, this.modules, this.connectionPool, this.security, this.server).register(builder).provides(this.getServiceName());
         for (Dependency dependency : this.threadPools.values()) {
             dependency.register(builder);
         }
@@ -180,7 +154,6 @@ public class RemoteCacheContainerConfigurationServiceConfigurator extends Capabi
         // Apply properties next, which may override formal security configuration
         builder.withProperties(this.properties)
                 .connectionTimeout(this.connectionTimeout)
-                .keySizeEstimate(this.keySizeEstimate)
                 .maxRetries(this.maxRetries)
                 .version(ProtocolVersion.parseVersion(this.protocolVersion))
                 .socketTimeout(this.socketTimeout)
@@ -192,18 +165,13 @@ public class RemoteCacheContainerConfigurationServiceConfigurator extends Capabi
                     .mBeanServerLookup((this.server != null) ? new MBeanServerProvider(this.server.get()) : null)
                 .tcpNoDelay(this.tcpNoDelay)
                 .tcpKeepAlive(this.tcpKeepAlive)
-                .valueSizeEstimate(this.valueSizeEstimate);
+                .transactionTimeout(this.transactionTimeout, TimeUnit.MILLISECONDS)
+                ;
 
         List<Module> modules = this.modules.get();
-        List<ClassLoader> loaders = new ArrayList<>(modules.size());
-        for (Module module : modules) {
-            loaders.add(module.getClassLoader());
-        }
-        ClassLoader loader = loaders.size() > 1 ? new AggregatedClassLoader(loaders) : loaders.get(0);
-        Marshaller marshaller = this.createMarshaller(loader);
-        InfinispanLogger.ROOT_LOGGER.debugf("%s cache-container will use %s", name, marshaller.getClass().getName());
+        Marshaller marshaller = this.marshallerFactory.apply(this.loader.get(), modules);
         builder.marshaller(marshaller);
-        builder.classLoader(loader);
+        builder.classLoader(modules.size() > 1 ? new AggregatedClassLoader(modules.stream().map(Module::getClassLoader).collect(Collectors.toList())) : modules.get(0).getClassLoader());
         builder.connectionPool().read(this.connectionPool.get());
         builder.asyncExecutorFactory().read(this.threadPools.get(ThreadPoolResourceDefinition.CLIENT).get());
 
@@ -225,17 +193,6 @@ public class RemoteCacheContainerConfigurationServiceConfigurator extends Capabi
             }
         }
 
-        builder.transaction().read(this.transaction.get());
-
         return builder.build();
     }
-
-    private Marshaller createMarshaller(ClassLoader loader) {
-        ModuleLoader moduleLoader = this.loader.get();
-        try {
-            return new ProtoStreamMarshaller(new ModuleClassLoaderMarshaller(moduleLoader), loader);
-        } catch (NoSuchElementException e) {
-            return new JBossMarshaller(ModularClassResolver.getInstance(moduleLoader), loader);
-        }
-    }
 }
